/**
 * @file aisl/instance.h
 *
 * Copyright (c) 2017-2019 by Löwenware Ltd.
 *
 * Project homepage: https://lowenware.com/aisl/
 *
 */

#ifndef AISL_INSTANCE_H_60576F41_454C_4189_B91A_F40501132230
#define AISL_INSTANCE_H_60576F41_454C_4189_B91A_F40501132230

#include <stdint.h>
#include <stdarg.h>
#include <aisl/types.h>
#include <aisl/config.h>



/**
 * @brief Allocates new AISL instance.
 *
 * @param cfg a pointer to #aisl_cfg_t structure.
 * @return an #aisl_t instance pointer.
 */
aisl_t
aisl_new( aisl_cfg_t cfg );


/**
 * @brief Frees previously allocated pointer of AISL instance.
 * @param instance a pointer to #aisl_t instance.
 */
void
aisl_free( aisl_t instance );


/** 
 * @brief A core function doing all the library routines.
 * Designed to be called inside application main loop
 * @param instance a pointer to #aisl_t instance.
 * @return #aisl_status_t code.
 */
aisl_status_t
aisl_run_cycle( aisl_t instance );


/** 
 * @brief Function to sleep CPU if nothing to do.
 * Calls select on all the opened sockets inside.
 * @param instance a pointer to #aisl_t instance.
 * @param usec a number of miliseconds to wait for any data on sockets.
 * @return #aisl_status_t code.
 */
aisl_status_t
aisl_sleep( aisl_t instance, uint32_t usec );


#endif /* !AISL_INSTANCE_H */
