/*
 * <aisl/types.h>
 *
 * Copyright (c) 2017-2019 by Löwenware Ltd.
 *
 * Project homepage: https://lowenware.com/aisl/
 *
 */

#ifndef AISL_TYPES_H_86A9DBA7_C0E6_4CF4_8A64_DAAD4A81031B
#define AISL_TYPES_H_86A9DBA7_C0E6_4CF4_8A64_DAAD4A81031B

#include <stdint.h>
#include <stdarg.h>
#include <stdbool.h>

#ifndef AISL_WITHOUT_SSL
#include <openssl/ssl.h>
#endif

#define AISL_AUTO_LENGTH (~0)

/* type casts */
#define AISL_CALLBACK(x) ((aisl_callback_t) x)


/* AISL Instance */
typedef struct aisl * aisl_t;

/* HTTP(s) Server */
typedef struct aisl_server * aisl_server_t;

/* HTTP(s) Client */
typedef struct aisl_client * aisl_client_t;

/* Server<->Client Stream */
typedef struct aisl_stream * aisl_stream_t;


/* status return codes */
typedef enum
{
    AISL_INPUT_ERROR    = -4
  , AISL_EXTCALL_ERROR  = -3
  , AISL_SYSCALL_ERROR  = -2
  , AISL_MALLOC_ERROR   = -1

  , AISL_SUCCESS        = 0
  , AISL_IDLE           = 1

} aisl_status_t;

#ifndef WITHOUT_STRINGIFIERS

const char *
aisl_status_to_string(aisl_status_t status);

#endif


/* Generic HTTP Enumerations */

typedef enum
{
    AISL_HTTP_0_9 = 0x0009
  , AISL_HTTP_1_0 = 0x0100
  , AISL_HTTP_1_1 = 0x0101
  , AISL_HTTP_2_0 = 0x0200

} aisl_http_version_t;


const char *
aisl_http_version_to_string( aisl_http_version_t version );


typedef enum
{
    AISL_HTTP_METHOD_UNKNOWN
  , AISL_HTTP_GET
  , AISL_HTTP_PUT
  , AISL_HTTP_POST
  , AISL_HTTP_HEAD
  , AISL_HTTP_TRACE
  , AISL_HTTP_DELETE
  , AISL_HTTP_OPTIONS
  , AISL_HTTP_CONNECT

  , AISL_HTTP_PRI

} aisl_http_method_t;


const char *
aisl_http_method_to_string( aisl_http_method_t method );


typedef enum 
{
    AISL_HTTP_CONTINUE                      = 100
  , AISL_HTTP_SWITCHING_PROTOCOLS

  , AISL_HTTP_OK                            = 200
  , AISL_HTTP_CREATED
  , AISL_HTTP_ACCEPTED
  , AISL_HTTP_NON_AUTHORITATIVE_INFORMATION
  , AISL_HTTP_NO_CONTENT
  , AISL_HTTP_RESET_CONTENT
  , AISL_HTTP_PARTIAL_CONTENT

  , AISL_HTTP_MULTIPLE_CHOICES              = 300
  , AISL_HTTP_MOVED_PERMANENTLY
  , AISL_HTTP_FOUND
  , AISL_HTTP_SEE_OTHER
  , AISL_HTTP_NOT_MODIFIED
  , AISL_HTTP_USE_PROXY
  , AISL_HTTP_UNUSED
  , AISL_HTTP_TEMPORARY_REDIRECT

  , AISL_HTTP_BAD_REQUEST                   = 400
  , AISL_HTTP_UNAUTHORIZED
  , AISL_HTTP_PAYMENT_REQUIRED
  , AISL_HTTP_FORBIDDEN
  , AISL_HTTP_NOT_FOUND
  , AISL_HTTP_METHOD_NOT_ALLOWED
  , AISL_HTTP_NOT_ACCEPTABLE
  , AISL_HTTP_PROXY_AUTHENTICATION_REQUIRED
  , AISL_HTTP_REQUEST_TIMEOUT
  , AISL_HTTP_CONFLICT
  , AISL_HTTP_GONE
  , AISL_HTTP_LENGTH_REQUIRED
  , AISL_HTTP_PRECONDITION_FAILED
  , AISL_HTTP_REQUEST_ENTITY_TOO_LARGE
  , AISL_HTTP_REQUEST_URI_TOO_LONG
  , AISL_HTTP_UNSUPPORTED_MEDIA_TYPE
  , AISL_HTTP_REQUESTED_RANGE_NOT_SATISFIABLE
  , AISL_HTTP_EXPECTATION_FAILED

  , AISL_HTTP_INTERNAL_SERVER_ERROR          = 500
  , AISL_HTTP_NOT_IMPLEMENTED
  , AISL_HTTP_BAD_GATEWAY
  , AISL_HTTP_SERVICE_UNAVAILABLE
  , AISL_HTTP_GATEWAY_TIMEOUT
  , AISL_HTTP_VERSION_NOT_SUPPORTED

} aisl_http_response_t;


const char *
aisl_http_response_to_string( aisl_http_response_t code );


/* Events */

typedef enum
{
    AISL_EVENT_SERVER_OPEN        = 100
  , AISL_EVENT_SERVER_ERROR       = 190

  , AISL_EVENT_CLIENT_CONNECT     = 200
  , AISL_EVENT_CLIENT_DISCONNECT  = 210

  , AISL_EVENT_STREAM_OPEN        = 300
  , AISL_EVENT_STREAM_HEADER      = 310
  , AISL_EVENT_STREAM_INPUT       = 320
  , AISL_EVENT_STREAM_REQUEST     = 330
  , AISL_EVENT_STREAM_OUTPUT      = 340
  , AISL_EVENT_STREAM_CLOSE       = 350
  , AISL_EVENT_STREAM_ERROR       = 390

} aisl_evt_code_t;


struct aisl_evt
{
  void            * source;
  aisl_evt_code_t   code;
  aisl_status_t     status;
};

typedef struct aisl_evt * aisl_evt_t;


/* void type event callback */
typedef void
(* aisl_callback_t) (aisl_evt_t const evt, void * p_ctx);


struct aisl_evt_stream_open
{
  struct aisl_evt      evt;
  const char         * path;
  const char         * query;
  aisl_http_method_t   http_method;
};

typedef struct aisl_evt_stream_open * aisl_evt_stream_open_t;


struct aisl_evt_stream_header
{
  struct aisl_evt      evt;
  const char         * key;
  const char         * value;
};

typedef struct aisl_evt_stream_header * aisl_evt_stream_header_t;


struct aisl_evt_stream_input
{
  struct aisl_evt      evt;
  const char         * data;
  int32_t              size;
};

typedef struct aisl_evt_stream_input * aisl_evt_stream_input_t;

#ifndef WITHOUT_STRINGIFIERS

const char *
aisl_evt_code_to_string( aisl_evt_code_t evt_code );

#endif

#endif /* !AISL_TYPES_H */
